@@ -1,17 +1,19 @@
 # ClamTk, copyright (C) 2004-2019 Dave M
 #
-# This file is part of ClamTk (https://dave-theunsub.github.io/clamtk).
+# This file is part of ClamTk
+# (https://gitlab.com/dave_m/clamtk-gtk3/).
 #
 # ClamTk is free software; you can redistribute it and/or modify it
 # under the terms of either:
 #
-# a) the GNU General Public License as published by the Free Software
-# Foundation; either version 1, or (at your option) any later version, or
+# a) the GNU General Public License as published by the
+# Free Software Foundation; either version 1, or (at your
+# option) any later version, or
 #
 # b) the "Artistic License".
 package ClamTk::GUI;
 
-use Gtk2 '-init';
+use Gtk3 '-init';
 use Glib 'TRUE', 'FALSE';
 
 # use strict;
@@ -32,95 +34,85 @@ sub start_gui {
     # Arch Linux, which may solve the issue - or at least stop dying
     # because it's missing an icon :|
     # https://aur.archlinux.org/packages/clamtk/
-    my $theme = Gtk2::IconTheme->get_default;
+    my $theme = Gtk3::IconTheme::get_default;
     $theme->append_search_path( '/usr/share/icons/gnome/24x24/actions' );
     $theme->append_search_path( '/usr/share/icons/gnome/24x24/places' );
     $theme->append_search_path( '/usr/share/icons/gnome/24x24/mimetypes' );
 
-    $window = Gtk2::Window->new;
+    $window = Gtk3::Window->new( 'toplevel' );
     $window->signal_connect(
         destroy => sub {
             $window->destroy;
-            Gtk2->main_quit;
+            Gtk3->main_quit;
             TRUE;
         }
     );
     $window->signal_connect(
         delete_event => sub {
             $window->destroy;
-            Gtk2->main_quit;
+            Gtk3->main_quit;
         }
     );
-    $window->set_title( _( 'Virus Scanner' ) );
     $window->set_border_width( 5 );
-    #$window->set_default_size( 340, 400 );
     $window->set_position( 'center' );
 
+    my $hb = Gtk3::HeaderBar->new;
+    $window->set_titlebar( $hb );
+
     my $images_dir = ClamTk::App->get_path( 'images' );
     my $pixbuf
-        = Gtk2::Gdk::Pixbuf->new_from_file_at_size( "$images_dir/clamtk.png",
+        = Gtk3::Gdk::Pixbuf->new_from_file_at_size( "$images_dir/clamtk.png",
         24, 24 );
 
-    if ( -e "$images_dir/clamtk.png" ) {
-        my $transparent = $pixbuf->add_alpha( TRUE, 0xff, 0xff, 0xff );
-        $window->set_icon( $transparent );
-    }
-
-    # my $white = Gtk2::Gdk::Color->new( 0xFFFF, 0xFFFF, 0xFFFF );
-    my $eb = Gtk2::EventBox->new;
-    #$eb->modify_bg( 'normal', $white );
+    my $eb = Gtk3::EventBox->new;
     $window->add( $eb );
 
-    $top_box = Gtk2::VBox->new( FALSE, 5 );
+    $top_box = Gtk3::Box->new( vertical, 5 );
+    $top_box->set_homogeneous( FALSE );
     $eb->add( $top_box );
 
-    my $toolbar = Gtk2::Toolbar->new;
-    $top_box->pack_start( $toolbar, FALSE, FALSE, 0 );
-    $toolbar->can_focus( FALSE );
-    $toolbar->set_show_arrow( FALSE );
-
-    my $image = Gtk2::Image->new;
+    my $image = Gtk3::Image->new;
     $image->set_from_pixbuf( $pixbuf );
 
-    my $button = Gtk2::ToolButton->new( $image, '' );
+    my $button = Gtk3::ToolButton->new( $image, '' );
     $button->set_sensitive( FALSE );
     $button->set_tooltip_text( _( 'ClamTk Virus Scanner' ) );
-    $toolbar->insert( $button, -1 );
+    $hb->pack_start( $button );
 
-    my $separator = Gtk2::SeparatorToolItem->new;
-    $separator->can_focus( TRUE );
+    $hb->set_title( _( 'Virus Scanner' ) );
+
+    my $separator = Gtk3::SeparatorToolItem->new;
     $separator->set_draw( FALSE );
     $separator->set_expand( TRUE );
-    $toolbar->insert( $separator, -1 );
+    $hb->pack_end( $separator );
 
-    $button = Gtk2::ToolButton->new_from_stock( 'gtk-about' );
-    $button->can_focus( FALSE );
-    $toolbar->insert( $button, -1 );
-    $button->set_tooltip_text( _( 'About' ) );
-    $button->signal_connect( clicked => \&about );
-
-    $toolbar->insert( Gtk2::SeparatorToolItem->new, -1 );
-
-    $button = Gtk2::ToolButton->new_from_stock( 'gtk-quit' );
-    $button->can_focus( FALSE );
-    $toolbar->insert( $button, -1 );
+    $button = Gtk3::ToolButton->new_from_stock( 'gtk-quit' );
     $button->set_tooltip_text( _( 'Quit' ) );
     $button->signal_connect(
         clicked => sub {
-            Gtk2->main_quit;
+            Gtk3->main_quit;
         }
     );
+    $hb->pack_end( $button );
 
-    $box = Gtk2::VBox->new( FALSE, 0 );
+    $hb->pack_end( Gtk3::SeparatorToolItem->new );
+
+    $button = Gtk3::ToolButton->new_from_stock( 'gtk-about' );
+    $button->set_tooltip_text( _( 'About' ) );
+    $button->signal_connect( clicked => \&about );
+    $hb->pack_end( $button );
+
+    $box = Gtk3::Box->new( vertical, 0 );
+    $top_box->set_homogeneous( FALSE );
     $box->set_border_width( 5 );
     $top_box->add( $box );
 
-    $infobar = Gtk2::InfoBar->new;
+    $infobar = Gtk3::InfoBar->new;
     $top_box->pack_start( $infobar, FALSE, FALSE, 0 );
     $infobar->add_button( 'gtk-go-back', -5 );
     $infobar->signal_connect( 'response' => \&add_default_view );
-    my $label = Gtk2::Label->new( '' );
-    $label->modify_font( Pango::FontDescription->from_string( 'Monospace' ) );
+
+    my $label = Gtk3::Label->new( '' );
     $label->set_use_markup( TRUE );
     $infobar->get_content_area()->add( $label );
     $infobar->grab_focus;
@@ -129,10 +121,10 @@ sub start_gui {
     my $ui_info = ClamTk::Shortcuts->get_ui_info;
     my @entries = ClamTk::Shortcuts->get_pseudo_keys;
 
-    my $actions = Gtk2::ActionGroup->new( 'Actions' );
+    my $actions = Gtk3::ActionGroup->new( 'Actions' );
     $actions->add_actions( \@entries, undef );
 
-    my $ui = Gtk2::UIManager->new;
+    my $ui = Gtk3::UIManager->new;
     $ui->insert_action_group( $actions, 0 );
 
     $window->add_accel_group( $ui->get_accel_group );
@@ -142,12 +134,13 @@ sub start_gui {
     startup();
 
     $window->show_all;
-    Gtk2->main;
+    Gtk3->main;
 }
 
 sub startup {
     # Updates available for gui or sigs outdated?
-    Gtk2->main_iteration while Gtk2->events_pending;
+    Gtk3::main_iteration while Gtk3::events_pending;
+
     my $startup_check = ClamTk::Startup->startup_check();
     my ( $message_type, $message );
     if ( $startup_check eq 'both' ) {
@@ -165,14 +158,12 @@ sub startup {
     }
     # Infobar is hidden typically, but if there
     # are updates, we need to show it
-    Gtk2->main_iteration while Gtk2->events_pending;
+    Gtk3::main_iteration while Gtk3::events_pending;
     set_infobar_mode( $message_type, $message );
     $window->queue_draw;
 
-    # $window->resize( 340, 400 );
     $infobar->show;
     $window->queue_draw;
-    Gtk2->main_iteration while Gtk2->events_pending;
 }
 
 sub set_infobar_mode {
@@ -180,9 +171,10 @@ sub set_infobar_mode {
 
     $infobar->set_message_type( $type );
     for my $c ( $infobar->get_content_area->get_children ) {
-        if ( $c->isa( 'Gtk2::Label' ) ) {
-            $c->modify_font(
-                Pango::FontDescription->from_string( 'Monospace' ) );
+        if ( $c->isa( 'Gtk3::Label' ) ) {
+            # Remove this (?)
+            # $c->modify_font(
+            #    Pango::FontDescription->from_string( 'Monospace' ) );
             $c->set_text( $text );
         }
     }
@@ -193,9 +185,9 @@ sub set_infobar_text_remote {
 
     $infobar->set_message_type( $type );
     for my $c ( $infobar->get_content_area->get_children ) {
-        if ( $c->isa( 'Gtk2::Label' ) ) {
-            $c->modify_font(
-                Pango::FontDescription->from_string( 'Monospace' ) );
+        if ( $c->isa( 'Gtk3::Label' ) ) {
+            # $c->modify_font(
+            #    Pango::FontDescription->from_string( 'Monospace' ) );
             $c->set_text( $text );
         }
     }
@@ -204,7 +196,7 @@ sub set_infobar_text_remote {
 sub add_configuration {
     my $show_this = shift;
 
-    my $label = Gtk2::Label->new;
+    my $label = Gtk3::Label->new;
   # $label->modify_font( Pango::FontDescription->from_string( 'Monospace' ) );
     $label->set_markup( "<b>$show_this</b>" );
     $label->set_alignment( 0.01, 0.5 );
@@ -213,12 +205,12 @@ sub add_configuration {
 }
 
 sub add_config_panels {
-    my $liststore = Gtk2::ListStore->new(
+    my $liststore = Gtk3::ListStore->new(
         # Link, Description, Tooltip
-        'Gtk2::Gdk::Pixbuf', 'Glib::String', 'Glib::String',
+        'Gtk3::Gdk::Pixbuf', 'Glib::String', 'Glib::String',
     );
 
-    my $view = Gtk2::IconView->new_with_model( $liststore );
+    my $view = Gtk3::IconView->new_with_model( $liststore );
     $view->set_can_focus( FALSE );
     $view->set_column_spacing( 10 );
     $view->set_columns( 4 );
@@ -241,7 +233,7 @@ sub add_config_panels {
             }
         }
     );
-     $view->signal_connect(
+    $view->signal_connect(
         'button-press-event' => sub {
             press( $path, $liststore );
         }
@@ -250,7 +242,7 @@ sub add_config_panels {
     my @data = (
         {   link        => _( 'Settings' ),
             description => _( 'View and set your preferences' ),
-            image       => 'gtk-preferences',
+            image       => 'preferences-system',
             button      => FALSE,
         },
         {   link        => _( 'Whitelist' ),
@@ -271,11 +263,11 @@ sub add_config_panels {
     );
 
     #<<<
-    my $theme = Gtk2::IconTheme->new;
+    my $theme = Gtk3::IconTheme->new;
     for my $item ( @data ) {
         my $iter = $liststore->append;
-        my $pix = Gtk2::IconTheme->get_default->load_icon(
-                $item->{image}, 24, 'use-builtin'
+        my $pix = Gtk3::IconTheme::get_default->load_icon(
+                 $item->{image}, 24, 'use-builtin'
         );
         $liststore->set( $iter,
                 0, $pix,
@@ -289,13 +281,13 @@ sub add_config_panels {
 }
 
 sub add_update_panels {
-    my $liststore = Gtk2::ListStore->new(
+    my $liststore = Gtk3::ListStore->new(
         # Link, Description, Tooltip
-        'Gtk2::Gdk::Pixbuf', 'Glib::String',
+        'Gtk3::Gdk::Pixbuf', 'Glib::String',
         'Glib::String',
     );
 
-    my $view = Gtk2::IconView->new_with_model( $liststore );
+    my $view = Gtk3::IconView->new_with_model( $liststore );
     $view->set_can_focus( FALSE );
     $view->set_column_spacing( 10 );
     $view->set_columns( 3 );
@@ -340,7 +332,7 @@ sub add_update_panels {
     #<<<
     for my $item ( @data ) {
         my $iter = $liststore->append;
-        my $pix = Gtk2::IconTheme->get_default->load_icon(
+        my $pix = Gtk3::IconTheme::get_default->load_icon(
                 $item->{image}, 24, 'use-builtin'
         );
         $liststore->set( $iter,
@@ -355,13 +347,13 @@ sub add_update_panels {
 }
 
 sub add_history_panels {
-    my $liststore = Gtk2::ListStore->new(
+    my $liststore = Gtk3::ListStore->new(
         # Link, Description, Tooltip
-        'Gtk2::Gdk::Pixbuf', 'Glib::String',
+        'Gtk3::Gdk::Pixbuf', 'Glib::String',
         'Glib::String',
     );
 
-    my $view = Gtk2::IconView->new_with_model( $liststore );
+    my $view = Gtk3::IconView->new_with_model( $liststore );
     $view->set_columns( 3 );
     $view->set_column_spacing( 10 );
     $view->set_row_spacing( 10 );
@@ -406,7 +398,7 @@ sub add_history_panels {
     #<<<
     for my $item ( @data ) {
         my $iter = $liststore->append;
-        my $pix = Gtk2::IconTheme->get_default->load_icon(
+        my $pix = Gtk3::IconTheme::get_default->load_icon(
                 $item->{image}, 24, 'use-builtin'
         );
         $liststore->set( $iter,
@@ -421,13 +413,13 @@ sub add_history_panels {
 }
 
 sub add_analysis_panels {
-    my $liststore = Gtk2::ListStore->new(
+    my $liststore = Gtk3::ListStore->new(
         # Link, Description, Tooltip
-        'Gtk2::Gdk::Pixbuf', 'Glib::String',
+        'Gtk3::Gdk::Pixbuf', 'Glib::String',
         'Glib::String',
     );
 
-    my $view = Gtk2::IconView->new_with_model( $liststore );
+    my $view = Gtk3::IconView->new_with_model( $liststore );
     $view->set_can_focus( FALSE );
     $view->set_column_spacing( 10 );
     $view->set_columns( 3 );
@@ -456,8 +448,7 @@ sub add_analysis_panels {
     );
 
     #<<<
-    # For CentOS only:
-    my @centos_data = (
+    my @scan_data = (
         {   link        => _( 'Scan a file' ),
             description => _( 'Scan a file' ),
             image       => 'gtk-file',
@@ -470,9 +461,9 @@ sub add_analysis_panels {
         },
     );
 
-    for my $item ( @centos_data ) {
+    for my $item ( @scan_data ) {
         my $iter = $liststore->append;
-        my $pix = Gtk2::IconTheme->get_default->load_icon(
+        my $pix = Gtk3::IconTheme::get_default->load_icon(
                 $item->{image}, 24, 'use-builtin'
         );
         $liststore->set( $iter,
@@ -482,7 +473,6 @@ sub add_analysis_panels {
         );
     }
 
-    # What should have been...
     my @data = (
         {   link        => _( 'Analysis' ),
             description => _( "View a file's reputation" ),
@@ -493,7 +483,7 @@ sub add_analysis_panels {
 
     for my $item ( @data ) {
         my $iter = $liststore->append;
-        my $pix = Gtk2::IconTheme->get_default->load_icon(
+        my $pix = Gtk3::IconTheme::get_default->load_icon(
                 $item->{image}, 24, 'use-builtin'
         );
         $liststore->set( $iter,
@@ -528,7 +518,7 @@ sub swap_button {
         for my $a ( $infobar->get_action_area ) {
             for my $b ( $a->get_children ) {
                 #$b->set_sensitive( $change_to );
-                if ( $b->isa( 'Gtk2::Button' ) ) {
+                if ( $b->isa( 'Gtk3::Button' ) ) {
                     $b->destroy;
                 }
             }
@@ -603,7 +593,7 @@ sub iconview_react {
 
 sub select_file {
     my $file   = '';
-    my $dialog = Gtk2::FileChooserDialog->new(
+    my $dialog = Gtk3::FileChooserDialog->new(
         _( 'Select a file' ), $window,
         'open',
         'gtk-cancel' => 'cancel',
@@ -616,7 +606,7 @@ sub select_file {
     $dialog->set_position( 'center-on-parent' );
     if ( 'ok' eq $dialog->run ) {
         $window->queue_draw;
-        Gtk2->main_iteration while ( Gtk2->events_pending );
+        Gtk3::main_iteration while Gtk3::events_pending;
         $file = $dialog->get_filename;
         $dialog->destroy;
         $window->queue_draw;
@@ -640,7 +630,7 @@ sub select_file {
 
 sub select_directory {
     my $directory = '';
-    my $dialog    = Gtk2::FileChooserDialog->new(
+    my $dialog    = Gtk3::FileChooserDialog->new(
         _( 'Select a directory' ), $window,
         'select-folder',
         'gtk-cancel' => 'cancel',
@@ -653,7 +643,7 @@ sub select_directory {
     }
     if ( 'ok' eq $dialog->run ) {
         $directory = $dialog->get_filename;
-        Gtk2->main_iteration while ( Gtk2->events_pending );
+        Gtk3::main_iteration while Gtk3::events_pending;
         $dialog->destroy;
         $window->queue_draw;
     } else {
@@ -691,19 +681,19 @@ sub add_default_view {
 
     $box->pack_start( add_config_panels(), TRUE, TRUE, 0 );
 
-    $box->pack_start( Gtk2::HSeparator->new, TRUE, TRUE, 2 );
+    $box->pack_start( Gtk3::HSeparator->new, TRUE, TRUE, 2 );
 
     $box->pack_start( add_configuration( _( 'History' ) ), TRUE, TRUE, 0 );
 
     $box->pack_start( add_history_panels(), TRUE, TRUE, 0 );
 
-    $box->pack_start( Gtk2::HSeparator->new, TRUE, TRUE, 2 );
+    $box->pack_start( Gtk3::Separator->new( 'horizontal' ), TRUE, TRUE, 2 );
 
     $box->pack_start( add_configuration( _( 'Updates' ) ), TRUE, TRUE, 0 );
 
     $box->pack_start( add_update_panels(), TRUE, TRUE, 0 );
 
-    $box->pack_start( Gtk2::HSeparator->new, TRUE, TRUE, 2 );
+    $box->pack_start( Gtk3::Separator->new( 'horizontal' ), TRUE, TRUE, 2 );
 
     $box->pack_start( add_configuration( _( 'Analysis' ) ), TRUE, TRUE, 0 );
 
@@ -712,14 +702,14 @@ sub add_default_view {
     $box->show_all;
     swap_button( FALSE );
 
-    Gtk2->main_iteration while Gtk2->events_pending;
+    Gtk3::main_iteration while Gtk3::events_pending;
     $window->resize( 340, 400 );
     $window->queue_draw;
-    Gtk2->main_iteration while Gtk2->events_pending;
+    Gtk3::main_iteration while Gtk3::events_pending;
 }
 
 sub about {
-    my $dialog = Gtk2::AboutDialog->new;
+    my $dialog = Gtk3::AboutDialog->new;
     my $license
         = 'ClamTk is free software; you can redistribute it and/or'
         . ' modify it under the terms of either:'
@@ -729,25 +719,24 @@ sub about {
         . ' b) the "Artistic License".';
     $dialog->set_wrap_license( TRUE );
     $dialog->set_position( 'mouse' );
-    $dialog->modify_font(
-        Pango::FontDescription->from_string( 'Monospace' ) );
+    # $dialog->modify_font(
+    #    Pango::FontDescription->from_string( 'Monospace' ) );
 
     my $images_dir = ClamTk::App->get_path( 'images' );
     my $icon       = "$images_dir/clamtk.png";
-    my $pixbuf     = Gtk2::Gdk::Pixbuf->new_from_file( $icon );
+    my $pixbuf     = Gtk3::Gdk::Pixbuf->new_from_file( $icon );
 
     $dialog->set_logo( $pixbuf );
     $dialog->set_version( ClamTk::App->get_TK_version() );
     $dialog->set_license( $license );
     $dialog->set_website_label( _( 'Homepage' ) );
-    $dialog->set_website( 'https://dave-theunsub.github.io/clamtk/' );
+    $dialog->set_website( 'https://gitlab.com/dave_m/clamtk/wikis/Home' );
     $dialog->set_logo( $pixbuf );
     $dialog->set_translator_credits(
-        'Please see the website for full listing' );
+        'Please see the credits.md for full listing' );
     $dialog->set_copyright( "\x{a9} Dave M 2004 - 2019" );
     $dialog->set_program_name( 'ClamTk' );
-    #$dialog->set_authors( [ 'Dave M', 'dave.nerd@gmail.com' ] );
-    $dialog->set_authors( 'Dave M <dave.nerd@gmail.com>' );
+    $dialog->set_authors( [ 'Dave M', 'dave.nerd@gmail.com' ] );
     $dialog->set_comments(
         _( 'ClamTk is a graphical front-end for Clam Antivirus' ) );
 
